@@ -67,13 +67,14 @@
 
   QJSValue supports the types defined in the \l{ECMA-262}
   standard: The primitive types, which are Undefined, Null, Boolean,
-  Number, and String; and the Object type. Additionally, built-in
+  Number, and String; and the Object and Array types. Additionally, built-in
   support is provided for Qt/C++ types such as QVariant and QObject.
 
   For the object-based types (including Date and RegExp), use the
   newT() functions in QJSEngine (e.g. QJSEngine::newObject())
   to create a QJSValue of the desired type. For the primitive types,
-  use one of the QJSValue constructor overloads.
+  use one of the QJSValue constructor overloads. For other types, e.g.
+  registered gadget types such as QPoint, you can use QJSEngine::toScriptValue.
 
   The methods named isT() (e.g. isBool(), isUndefined()) can be
   used to test if a value is of a certain type. The methods named
@@ -107,6 +108,38 @@
   script code, or QJSValueIterator in C++.
 
   \sa QJSEngine, QJSValueIterator
+
+  \section1 Working With Arrays
+
+  To create an array using QJSValue, use \l QJSEngine::newArray():
+
+  \code
+  // Assumes that this class was declared in QML.
+  QJSValue jsArray = engine->newArray(3);
+  \endcode
+
+  To set individual elements in the array, use
+  the \l {QJSValue::}{setProperty(quint32 arrayIndex, const QJSValue &value)}
+  overload. For example, to fill the array above with integers:
+
+  \code
+  for (int i = 0; i < 3; ++i) {
+      jsArray.setProperty(i, QRandomGenerator::global().generate());
+  }
+  \endcode
+
+  To determine the length of the array, access the \c "length" property.
+  To access array elements, use the
+  \l {QJSValue::}{property(quint32 arrayIndex)} overload. The following code
+  reads the array we created above back into a list:
+
+  \code
+  QVector<int> integers;
+  const int length = jsArray.property("length").toInt();
+  for (int i = 0; i < length; ++i) {
+      integers.append(jsArray.property(i).toInt());
+  }
+  \endcode
 */
 
 /*!
@@ -178,7 +211,7 @@ QJSValue::QJSValue(SpecialValue value)
     : d(0)
 {
     if (value == NullValue)
-        QJSValuePrivate::setVariant(this, QVariant(QMetaType::VoidStar, (void *)0));
+        QJSValuePrivate::setVariant(this, QVariant::fromValue(nullptr));
 }
 
 /*!
@@ -293,7 +326,10 @@ bool QJSValue::isNull() const
     if (val)
         return val->isNull();
     QVariant *variant = QJSValuePrivate::getVariant(this);
-    return variant && variant->userType() == QMetaType::VoidStar;
+    if (!variant)
+        return false;
+    const int type = variant->userType();
+    return type == QMetaType::Nullptr || type == QMetaType::VoidStar;
 }
 
 /*!
@@ -582,7 +618,7 @@ quint32 QJSValue::toUInt() const
     \table
     \header \li Input Type \li Result
     \row    \li Undefined  \li An invalid QVariant.
-    \row    \li Null       \li A QVariant containing a null pointer (QMetaType::VoidStar).
+    \row    \li Null       \li A QVariant containing a null pointer (QMetaType::Nullptr).
     \row    \li Boolean    \li A QVariant containing the value of the boolean.
     \row    \li Number     \li A QVariant containing the value of the number.
     \row    \li String     \li A QVariant containing the value of the string.
@@ -609,8 +645,8 @@ QVariant QJSValue::toVariant() const
     if (Object *o = val->as<Object>())
         return o->engine()->toVariant(*val, /*typeHint*/ -1, /*createJSValueForObjects*/ false);
 
-    if (val->isString())
-        return QVariant(val->stringValue()->toQString());
+    if (String *s = val->stringValue())
+        return QVariant(s->toQString());
     if (val->isBoolean())
         return QVariant(val->booleanValue());
     if (val->isNumber()) {
@@ -619,7 +655,7 @@ QVariant QJSValue::toVariant() const
         return QVariant(val->asDouble());
     }
     if (val->isNull())
-        return QVariant(QMetaType::VoidStar, 0);
+        return QVariant(QMetaType::Nullptr, 0);
     Q_ASSERT(val->isUndefined());
     return QVariant();
 }
@@ -663,11 +699,11 @@ QJSValue QJSValue::call(const QJSValueList &args)
         callData->args[i] = QJSValuePrivate::convertedToValue(engine, args.at(i));
     }
 
-    ScopedValue result(scope, f->call(callData));
+    f->call(scope, callData);
     if (engine->hasException)
-        result = engine->catchException();
+        scope.result = engine->catchException();
 
-    return QJSValue(engine, result->asReturnedValue());
+    return QJSValue(engine, scope.result.asReturnedValue());
 }
 
 /*!
@@ -719,11 +755,11 @@ QJSValue QJSValue::callWithInstance(const QJSValue &instance, const QJSValueList
         callData->args[i] = QJSValuePrivate::convertedToValue(engine, args.at(i));
     }
 
-    ScopedValue result(scope, f->call(callData));
+    f->call(scope, callData);
     if (engine->hasException)
-        result = engine->catchException();
+        scope.result = engine->catchException();
 
-    return QJSValue(engine, result->asReturnedValue());
+    return QJSValue(engine, scope.result.asReturnedValue());
 }
 
 /*!
@@ -767,11 +803,11 @@ QJSValue QJSValue::callAsConstructor(const QJSValueList &args)
         callData->args[i] = QJSValuePrivate::convertedToValue(engine, args.at(i));
     }
 
-    ScopedValue result(scope, f->construct(callData));
+    f->construct(scope, callData);
     if (engine->hasException)
-        result = engine->catchException();
+        scope.result = engine->catchException();
 
-    return QJSValue(engine, result->asReturnedValue());
+    return QJSValue(engine, scope.result.asReturnedValue());
 }
 
 #ifdef QT_DEPRECATED
@@ -882,14 +918,14 @@ QJSValue& QJSValue::operator=(const QJSValue& other)
 
 static bool js_equal(const QString &string, const QV4::Value &value)
 {
-    if (value.isString())
-        return string == value.stringValue()->toQString();
+    if (String *s = value.stringValue())
+        return string == s->toQString();
     if (value.isNumber())
         return RuntimeHelpers::stringToNumber(string) == value.asDouble();
     if (value.isBoolean())
         return RuntimeHelpers::stringToNumber(string) == double(value.booleanValue());
-    if (value.isObject()) {
-        Scope scope(value.objectValue()->engine());
+    if (Object *o = value.objectValue()) {
+        Scope scope(o->engine());
         ScopedValue p(scope, RuntimeHelpers::toPrimitive(value, PREFERREDTYPE_HINT));
         return js_equal(string, p);
     }
@@ -938,7 +974,7 @@ bool QJSValue::equals(const QJSValue& other) const
     if (!ov)
         return other.equals(*this);
 
-    return Runtime::compareEqual(*v, *ov);
+    return Runtime::method_compareEqual(*v, *ov);
 }
 
 /*!
@@ -976,8 +1012,8 @@ bool QJSValue::strictlyEquals(const QJSValue& other) const
             return *variant == *QJSValuePrivate::getVariant(&other);
         if (variant->type() == QVariant::Map || variant->type() == QVariant::List)
             return false;
-        if (ov->isString())
-            return variant->toString() == ov->stringValue()->toQString();
+        if (String *s = ov->stringValue())
+            return variant->toString() == s->toQString();
         return false;
     }
     if (!ov)
@@ -997,6 +1033,10 @@ bool QJSValue::strictlyEquals(const QJSValue& other) const
   occurred, property() returns the value that was thrown (typically
   an \c{Error} object).
 
+  To access array elements, use the
+  \l {QJSValue::}{setProperty(quint32 arrayIndex, const QJSValue &value)}
+  overload instead.
+
   \sa setProperty(), hasProperty(), QJSValueIterator
 */
 QJSValue QJSValue::property(const QString& name) const
@@ -1015,7 +1055,7 @@ QJSValue QJSValue::property(const QString& name) const
     if (idx < UINT_MAX)
         return property(idx);
 
-    s->makeIdentifier(engine);
+    s->makeIdentifier();
     QV4::ScopedValue result(scope, o->get(s));
     if (engine->hasException)
         result = engine->catchException();
@@ -1028,8 +1068,25 @@ QJSValue QJSValue::property(const QString& name) const
 
   Returns the property at the given \a arrayIndex.
 
-  This function is provided for convenience and performance when
-  working with array objects.
+  It is possible to access elements in an array in two ways. The first is to
+  use the array index as the property name:
+
+  \code
+  qDebug() << jsValueArray.property(QLatin1String("4")).toString();
+  \endcode
+
+  The second is to use the overload that takes an index:
+
+  \code
+  qDebug() << jsValueArray.property(4).toString();
+  \endcode
+
+  Both of these approaches achieve the same result, except that the latter:
+
+  \list
+  \li Is easier to use (can use an integer directly)
+  \li Is faster (no conversion to integer)
+  \endlist
 
   If this QJSValue is not an Array object, this function behaves
   as if property() was called with the string representation of \a
@@ -1061,6 +1118,10 @@ QJSValue QJSValue::property(quint32 arrayIndex) const
   If this QJSValue does not already have a property with name \a name,
   a new property is created.
 
+  To modify array elements, use the
+  \l {QJSValue::}{setProperty(quint32 arrayIndex, const QJSValue &value)}
+  overload instead.
+
   \sa property(), deleteProperty()
 */
 void QJSValue::setProperty(const QString& name, const QJSValue& value)
@@ -1086,7 +1147,7 @@ void QJSValue::setProperty(const QString& name, const QJSValue& value)
         return;
     }
 
-    s->makeIdentifier(scope.engine);
+    s->makeIdentifier();
     QV4::ScopedValue v(scope, QJSValuePrivate::convertedToValue(engine, value));
     o->put(s, v);
     if (engine->hasException)
@@ -1098,12 +1159,31 @@ void QJSValue::setProperty(const QString& name, const QJSValue& value)
 
   Sets the property at the given \a arrayIndex to the given \a value.
 
-  This function is provided for convenience and performance when
-  working with array objects.
+  It is possible to modify elements in an array in two ways. The first is to
+  use the array index as the property name:
+
+  \code
+  jsValueArray.setProperty(QLatin1String("4"), value);
+  \endcode
+
+  The second is to use the overload that takes an index:
+
+  \code
+  jsValueArray.setProperty(4, value);
+  \endcode
+
+  Both of these approaches achieve the same result, except that the latter:
+
+  \list
+  \li Is easier to use (can use an integer directly)
+  \li Is faster (no conversion to integer)
+  \endlist
 
   If this QJSValue is not an Array object, this function behaves
   as if setProperty() was called with the string representation of \a
   arrayIndex.
+
+  \sa {QJSValue::}{property(quint32 arrayIndex)}, {Working With Arrays}
 */
 void QJSValue::setProperty(quint32 arrayIndex, const QJSValue& value)
 {
@@ -1233,6 +1313,28 @@ QObject *QJSValue::toQObject() const
     return wrapper->object();
 }
 
+/*!
+  \since 5.8
+
+ * If this QJSValue is a QMetaObject, returns the QMetaObject pointer
+ * that the QJSValue represents; otherwise, returns 0.
+ *
+ * \sa isQMetaObject()
+ */
+const QMetaObject *QJSValue::toQMetaObject() const
+{
+    QV4::ExecutionEngine *engine = QJSValuePrivate::engine(this);
+    if (!engine)
+        return 0;
+    QV4::Scope scope(engine);
+    QV4::Scoped<QV4::QMetaObjectWrapper> wrapper(scope, QJSValuePrivate::getValue(this));
+    if (!wrapper)
+        return 0;
+
+    return wrapper->metaObject();
+}
+
+
 /*!
   Returns a QDateTime representation of this value, in local time.
   If this QJSValue is not a date, or the value of the date is NaN
@@ -1286,4 +1388,18 @@ bool QJSValue::isQObject() const
     return val && val->as<QV4::QObjectWrapper>() != 0;
 }
 
+/*!
+  \since 5.8
+
+  Returns true if this QJSValue is a QMetaObject; otherwise returns
+  false.
+
+  \sa toQMetaObject(), QJSEngine::newQMetaObject()
+*/
+bool QJSValue::isQMetaObject() const
+{
+    QV4::Value *val = QJSValuePrivate::getValue(this);
+    return val && val->as<QV4::QMetaObjectWrapper>() != 0;
+}
+
 QT_END_NAMESPACE
