@@ -4,15 +4,14 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch08/8.12/8.12.1/8.12.1-1_3.js
- * @description Properties - [[HasOwnProperty]] (old style inherited property)
+ * @path ch08/8.12/8.12.1/8.12.1-1_2.js
+ * @description Properties - [[HasOwnProperty]] (old style own property)
  */
 
 function testcase() {
 
-    var base = {foo:42};
-    var o = Object.create(base);
-    return o.hasOwnProperty("foo")===false;
+    var o = {foo: 42};
+    return o.hasOwnProperty("foo");
 
 }
 runTestCase(testcase);
