@@ -4,12 +4,18 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.12/15.12.3/15.12.3-11-12.js
- * @description A JSON.stringify replacer function applied to a top level scalar can return an Array.
+ * @path ch15/15.12/15.12.3/15.12.3_4-1-3.js
+ * @description JSON.stringify a indirectly circular object throws a error
  */
 
 
 function testcase() {
-  return JSON.stringify(42, function(k, v) { return v==42 ?[4,2]:v }) === '[4,2]';
+  var obj = {p1: {p2: {}}};
+  obj.p1.p2.prop = obj;
+  try {
+     JSON.stringify(obj);
+     return false;  // should not reach here
+     }
+   catch (e) {return  true}
   }
 runTestCase(testcase);
