@@ -4,14 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.2/15.2.3/15.2.3.13/15.2.3.13-2-20.js
- * @description Object.isExtensible returns true for all built-in objects (RegExp.prototype)
+ * @path ch15/15.2/15.2.3/15.2.3.3/15.2.3.3-4-129.js
+ * @description Object.getOwnPropertyDescriptor returns data desc for functions on built-ins (Date.prototype.getUTCMonth)
  */
 
 
 function testcase() {
-  var e = Object.isExtensible(RegExp.prototype);
-  if (e === true) {
+  var desc = Object.getOwnPropertyDescriptor(Date.prototype, "getUTCMonth");
+  if (desc.value === Date.prototype.getUTCMonth &&
+      desc.writable === true &&
+      desc.enumerable === false &&
+      desc.configurable === true) {
     return true;
   }
  }
