@@ -4,42 +4,38 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.19/15.4.4.19-8-c-i-14.js
- * @description Array.prototype.map - element to be retrieved is own accessor property that overrides an inherited accessor property on an Array
+ * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-i-14.js
+ * @description  Array.prototype.every - element to be retrieved is own accessor property that overrides an inherited accessor property on an Array
  */
 
 
 function testcase() {
 
-        var kValue = "abc";
-
         function callbackfn(val, idx, obj) {
             if (idx === 0) {
-                return val === kValue;
+                return val === 5;
+            } else {
+                return true;
             }
-            return false;
         }
 
         var arr = [];
-
         try {
             Object.defineProperty(Array.prototype, "0", {
                 get: function () {
-                    return 11;
+                    return 5;
                 },
                 configurable: true
             });
 
             Object.defineProperty(arr, "0", {
                 get: function () {
-                    return kValue;
+                    return 11;
                 },
                 configurable: true
             });
 
-            var testResult = arr.map(callbackfn);
-
-            return testResult[0] === true;
+            return !arr.every(callbackfn);
         } finally {
             delete Array.prototype[0];
         }
