@@ -4,15 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.20/15.4.4.20-9-c-i-16.js
- * @description Array.prototype.filter - element to be retrieved is inherited accessor property on an Array
+ * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-i-16.js
+ * @description Array.prototype.every - element to be retrieved is inherited accessor property on an Array
  */
 
 
 function testcase() {
 
         function callbackfn(val, idx, obj) {
-            return idx === 0 && val === 11;
+            if (idx === 0) {
+                return val !== 11;
+            } else {
+                return true;
+            }
         }
 
         try {
@@ -22,9 +26,8 @@ function testcase() {
                 },
                 configurable: true
             });
-            var newArr = [, , , ].filter(callbackfn);
 
-            return newArr.length === 1 && newArr[0] === 11;
+            return ![, , , ].every(callbackfn);
         } finally {
             delete Array.prototype[0];
         }
