@@ -4,31 +4,30 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.18/15.4.4.18-7-c-ii-7.js
- * @description Array.prototype.forEach - unhandled exceptions happened in callbackfn terminate iteration
+ * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-ii-7.js
+ * @description Array.prototype.every - unhandled exceptions happened in callbackfn terminate iteration
  */
 
 
 function testcase() {
 
-        var accessed = false;
+        var called = 0;
 
         function callbackfn(val, idx, obj) {
-            if (idx > 0) {
-                accessed = true;
-            }
-            if (idx === 0) {
+            called++;
+            if (called === 1) {
                 throw new Error("Exception occurred in callbackfn");
             }
+            return true;
         }
 
         var obj = { 0: 11, 4: 10, 10: 8, length: 20 };
 
         try {
-            Array.prototype.forEach.call(obj, callbackfn);
+            Array.prototype.every.call(obj, callbackfn);
             return false;
         } catch (ex) {
-            return ex instanceof Error && !accessed;
+            return 1 === called;
         }
     }
 runTestCase(testcase);
