@@ -4,8 +4,8 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.20/15.4.4.20-9-c-iii-8.js
- * @description Array.prototype.filter - return value of callbackfn is a nunmber (value is -0)
+ * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-iii-8.js
+ * @description Array.prototype.every - return value of callbackfn is a number (value is positive number)
  */
 
 
@@ -15,10 +15,9 @@ function testcase() {
 
         function callbackfn(val, idx, obj) {
             accessed = true;
-            return -0;
+            return 5;
         }
 
-        var newArr = [11].filter(callbackfn);
-        return newArr.length === 0 && accessed;
+        return [11].every(callbackfn) && accessed;
     }
 runTestCase(testcase);
