@@ -4,23 +4,23 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-iii-2.js
- * @description Array.prototype.every - return value of callbackfn is null
+ * @path ch15/15.4/15.4.4/15.4.4.20/15.4.4.20-9-c-iii-2.js
+ * @description Array.prototype.filter - return value of callbackfn is undefined
  */
 
 
 function testcase() {
 
         var accessed = false;
-        var obj = { 0: 11, length: 1 };
 
-        function callbackfn(val, idx, obj) {
+        function callbackfn(val, idx, o) {
             accessed = true;
-            return null;
+            return undefined;
         }
 
-        
+        var obj = { 0: 11, length: 1 };
 
-        return !Array.prototype.every.call(obj, callbackfn) && accessed;
+        var newArr = Array.prototype.filter.call(obj, callbackfn);
+        return  newArr.length === 0 && accessed;
     }
 runTestCase(testcase);
